@@ -1,10 +1,11 @@
-from __future__ import unicode_literals
+from typing import Callable, Iterable, List, Mapping, Optional, Pattern, Union
 
-from six import string_types
-from prompt_toolkit.completion import Completer, Completion
+from prompt_toolkit.completion import CompleteEvent, Completer, Completion
+from prompt_toolkit.document import Document
+from prompt_toolkit.formatted_text import AnyFormattedText
 
 __all__ = [
-    'WordCompleter',
+    "WordCompleter",
 ]
 
 
@@ -23,20 +24,37 @@ class WordCompleter(Completer):
         contain spaces. (Can not be used together with the WORD option.)
     :param match_middle: When True, match not only the start, but also in the
                          middle of the word.
+    :param pattern: Optional compiled regex for finding the word before
+        the cursor to complete. When given, use this regex pattern instead of
+        default one (see document._FIND_WORD_RE)
     """
-    def __init__(self, words, ignore_case=False, meta_dict=None, WORD=False,
-                 sentence=False, match_middle=False):
+
+    def __init__(
+        self,
+        words: Union[List[str], Callable[[], List[str]]],
+        ignore_case: bool = False,
+        display_dict: Optional[Mapping[str, AnyFormattedText]] = None,
+        meta_dict: Optional[Mapping[str, AnyFormattedText]] = None,
+        WORD: bool = False,
+        sentence: bool = False,
+        match_middle: bool = False,
+        pattern: Optional[Pattern[str]] = None,
+    ) -> None:
+
         assert not (WORD and sentence)
-        assert callable(words) or all(isinstance(w, string_types) for w in words)
 
         self.words = words
         self.ignore_case = ignore_case
+        self.display_dict = display_dict or {}
         self.meta_dict = meta_dict or {}
         self.WORD = WORD
         self.sentence = sentence
         self.match_middle = match_middle
+        self.pattern = pattern
 
-    def get_completions(self, document, complete_event):
+    def get_completions(
+        self, document: Document, complete_event: CompleteEvent
+    ) -> Iterable[Completion]:
         # Get list of words.
         words = self.words
         if callable(words):
@@ -46,12 +64,14 @@ class WordCompleter(Completer):
         if self.sentence:
             word_before_cursor = document.text_before_cursor
         else:
-            word_before_cursor = document.get_word_before_cursor(WORD=self.WORD)
+            word_before_cursor = document.get_word_before_cursor(
+                WORD=self.WORD, pattern=self.pattern
+            )
 
         if self.ignore_case:
             word_before_cursor = word_before_cursor.lower()
 
-        def word_matches(word):
+        def word_matches(word: str) -> bool:
             """ True when the word before the cursor matches. """
             if self.ignore_case:
                 word = word.lower()
@@ -63,5 +83,11 @@ class WordCompleter(Completer):
 
         for a in words:
             if word_matches(a):
-                display_meta = self.meta_dict.get(a, '')
-                yield Completion(a, -len(word_before_cursor), display_meta=display_meta)
+                display = self.display_dict.get(a, a)
+                display_meta = self.meta_dict.get(a, "")
+                yield Completion(
+                    a,
+                    -len(word_before_cursor),
+                    display=display,
+                    display_meta=display_meta,
+                )
