@@ -26,6 +26,7 @@ __all__ = [
     "is_conemu_ansi",
     "is_windows",
     "in_main_thread",
+    "get_bell_environment_variable",
     "get_term_environment_variable",
     "take_using_weights",
     "to_str",
@@ -210,6 +211,14 @@ def in_main_thread() -> bool:
     return threading.current_thread().__class__.__name__ == "_MainThread"
 
 
+def get_bell_environment_variable() -> bool:
+    """
+    True if env variable is set to true (true, TRUE, TrUe, 1).
+    """
+    value = os.environ.get("PROMPT_TOOLKIT_BELL", "true")
+    return value.lower() in ("1", "true")
+
+
 def get_term_environment_variable() -> str:
     " Return the $TERM environment variable. "
     return os.environ.get("TERM", "")
@@ -307,6 +316,6 @@ def is_dumb_terminal(term: Optional[str] = None) -> bool:
     without cursor positioning and color support.
     """
     if term is None:
-        term = os.environ.get("TERM", "")
+        return is_dumb_terminal(os.environ.get("TERM", ""))
 
-    return term in ["dumb", "unknown"]
+    return term.lower() in ["dumb", "unknown"]
